const path = require('path')
const pluginsInterface = require('structure-plugins')
const Router = require('structure-router')
const Server = require('../../index')

/** @test {Server} */
describe('HTTP Server', function() {

  it('should initialize', function() {

    var api = new Server({
      sockets: false
    })

    expect(api).to.be.an('object')
    expect(api.logRequestInfo).to.be.a('function')
    expect(api.server).to.be.an('function') // Express server
    expect(api.router).to.be.an('object')
    expect(api.start).to.be.a('function')
    expect(api.stop).to.be.a('function')
    expect(api.use).to.be.a('function')

  })

  /** @test {Server#start} */
  it('should start', function() {

    var api = new Server({
      sockets: false
    })
    sinon.stub(api.server, 'listen').returns(api.server)

    api.start()

    expect(api.server.listen.calledOnce).to.be.true

  })

  /** @test {Server#stop} */
  it('should stop', function() {

    var api = new Server({
      sockets: false
    })
    sinon.stub(api.server, 'listen').returns(api.server)
    sinon.stub(api, 'stop')

    api.start()
    api.stop()

    expect(api.stop.calledOnce).to.be.true

  })

  it.skip('should prevent query parameter pollution', async function() {

    var pkg = {
      title: 'My Template'
    }

    var res = await new MockHTTPServer()
      .post(`/api/v${process.env.API_VERSION}/templates?desc=cool&desc=cooler`)
      .send(pkg)

    expect(res.body.req.queryPolluted).to.be.true
    expect(res.body.req.query.desc).to.equal('cooler')

  })

  it.skip('should generate routes list', function() {

    const loaded = pluginsInterface({
      Controller: class RootController {},
      Model: class RootModel {},
      list: [
        path.join(__dirname, '../helpers/plugin1')
      ]
    })

    const api = new Server({
      router: new Router({
        routes: loaded.routes
      })
    })

    expect(api.routesList.length > 0).to.be.true

  })

})
